from typing import Dict, List, Self

from pydantic import ValidationError as PydanticValidationError

from pytempl.errors import ValidationError
from pytempl.utils import (
    ValidatorFunction,
    format_validation_error_message,
    validate_dictionary_data,
)

from ._base import (
    BaseHTMLElement,
    GlobalHTMLAttributes,
)

try:
    from typing import Unpack
except ImportError:
    from typing_extensions import Unpack


class BodyAttributes(GlobalHTMLAttributes):
    @classmethod
    def validate(
        cls,
        data: Dict,
        default_values: Dict | None = None,
        custom_validators: List[ValidatorFunction] | None = None,
    ) -> Self:
        return validate_dictionary_data(cls, data, default_values, custom_validators)


class Body(BaseHTMLElement):
    tag_name = "body"
    have_children = True

    def __init__(self, **attributes: Unpack[BodyAttributes]):
        try:
            validated_attributes = BodyAttributes.validate(attributes)
        except (ValidationError, PydanticValidationError) as err:
            raise ValueError(format_validation_error_message(err))

        super().__init__(**validated_attributes)
